package com.xydl.cac.util;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.xydl.cac.entity.IcdConfigType;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

public class IcdXmlUtil {

    public static LinkedHashMap<String, IcdConfigType> loadIcdType(String xml) throws Exception {
        XmlMapper xmlMapper = XmlMapper.builder()
                .build();
        JsonNode root = xmlMapper.readTree(xml);
        LinkedHashMap<String, IcdConfigType> result = processTypeRoot(root);
        return result;
    }

    private static LinkedHashMap<String, IcdConfigType> processTypeRoot(JsonNode root) {
        LinkedHashMap<String, IcdConfigType> result = new LinkedHashMap<>();

        List<JsonNode> iedList = findNodes(root, "IED");
        for (JsonNode iedNode : iedList) {
            processIEDNode(iedNode, result);
        }
        return result;
    }

    private static void processIEDNode(JsonNode iedNode, LinkedHashMap<String, IcdConfigType> result) {
        String iedName = iedNode.get("name").asText();

        List<JsonNode> devList = findNodes(iedNode, "LDevice");
        for (JsonNode dev : devList) {
            processTypeDeviceNode(dev, iedName, result);
        }
    }

    private static void processTypeDeviceNode(JsonNode deviceNode, String iedName, LinkedHashMap<String, IcdConfigType> result) {
        String ldeviceInst = deviceNode.get("inst").asText();
        Map<String, JsonNode> lnMap = buildLNMap(deviceNode);

        List<JsonNode> fcdaList = findNodes(deviceNode, "FCDA");
        for (JsonNode fcdaNode : fcdaList) {
            String lnClass = fcdaNode.get("lnClass").asText();
            String lnInst = fcdaNode.get("lnInst").asText();
            String doName = fcdaNode.get("doName").asText();
            String fc = fcdaNode.get("fc").asText();

            JsonNode lnNode = lnMap.get(lnClass + lnInst);
            String dai = findLN_DOI_DAI(lnNode, doName);

            String key = iedName + ldeviceInst + "/" + lnClass;
            String param = fc + "$" + doName + "$" + dai;
            if ("MX".equals(fc)) {
                IcdConfigType config = result.get(key);
                if (config == null) {
                    config = IcdConfigType.builder()
                            .iedName(iedName)
                            .ldeviceInst(ldeviceInst)
                            .lnClass(lnClass)
                            .build();
                    result.put(key, config);
                }
                config.addInst(lnInst);
                config.addAtt(doName, dai);
            }
        }
    }

    private static Map<String, JsonNode> buildLNMap(JsonNode deviceNode) {
        Map<String, JsonNode> map = new LinkedHashMap<>();
        List<JsonNode> lnList = findNodes(deviceNode, "LN");
        for (JsonNode lnNode : lnList) {
            String lnClass = lnNode.get("lnClass").asText();
            String inst = lnNode.get("inst").asText();
            map.put(lnClass + inst, lnNode);
        }
        return map;
    }

    private static String findLN_DOI_DAI(JsonNode lnNode, String doName) {
        String result = "";
        List<JsonNode> doiList = findNodes(lnNode, "DOI");
        for (JsonNode doiNode : doiList) {
            String doiName = doiNode.get("name").asText();
            if (doiName.equals(doName)) {
                JsonNode pnode = doiNode;
                List<JsonNode> sdiList = findNodes(doiNode, "SDI");
                if (sdiList.size() > 0) {
                    JsonNode sdiNode = sdiList.get(0);
                    result = sdiNode.get("name").asText();
                    pnode = sdiNode;
                }
                List<JsonNode> daiList = findNodes(pnode, "DAI");
                if (daiList.size() > 0) {
                    JsonNode daiNode = daiList.get(0);
                    String daiName = daiNode.get("name").asText();
                    if (StringUtils.isNotBlank(result)) {
                        result = result + "$" + daiName;
                    } else {
                        result = daiName;
                    }
                }
                break;
            }
        }
        return result;
    }

    private static List<JsonNode> findNodes(JsonNode root, String fieldName) {
        List<JsonNode> result = new ArrayList<>();
        List<JsonNode> list = root.findValues(fieldName);
        for (JsonNode node : list) {
            if (node instanceof ObjectNode) {
                result.add(node);
            }
            if (node instanceof ArrayNode) {
                ArrayNode array = (ArrayNode) node;
                Iterator<JsonNode> elements = array.elements();
                while (elements.hasNext()) {
                    JsonNode item = elements.next();
                    result.add(item);
                }
            }
        }
        return result;
    }
}
